var     hashFn = require('string-hash')
    ,   parseDotnotation = require('dotnotater');

function CreateTrackingObject(array, keys) {
    var result = {}, hash;
    if (array.length > 0) {
        array.forEach(function (item) {

            var keyString = "";
            if (keys != undefined && keys.length > 0) { //this is a keyed obj
                keys.forEach(function (key) {
                    keyString += JSON.stringify(item[key]);
                })
                hash = hashFn(keyString)

                if (result.hasOwnProperty(hash)) { //don't allow dupes on keyed fields
                    throw new Error("Duplicate key")
                } else {
                    result[hash] = {};
                    result[hash].item  = item;
                    result[hash].occurrences = 1;
                }
                return result;
            };


            hash = hashFn(JSON.stringify(item));
            if (result.hasOwnProperty(hash)) { //check if the object is a dupe.
                result[hash].occurrences++;

            } else {  //not a dupe
                result[hash] = {};
                result[hash].item = item;
                result[hash].occurrences = 1;
            }
        });
    };
    return result;

}

function ArrayTracker(options) {
    var self = this;

    function tracker() {
        var self = this;
        var keys, orgArray, context, objKeys, initialized = false;

        self.isInitialized = initialized;

        self.initialize = function (array, pathToArray, keys) {
            var inputArray;
            objKeys = keys;
            if (Object.prototype.toString.call(array) !== '[object Array]' && pathToArray === undefined && (typeof pathToArray !== 'string')) {
                throw new Error("array argument is not an array!");
            };

            if (Object.prototype.toString.call(pathToArray) === '[object Array]') {
                objKeys = pathToArray;
            }
            if (typeof array === "object" && pathToArray != objKeys) {
                inputArray = parseDotnotation(array, pathToArray)
            } else {
                inputArray = array;
            };

            orgArray = CreateTrackingObject(inputArray, objKeys);

            if (orgArray != undefined) {
                initialized = true;
                return initialized;
            } else {
                return initialized;
            };
        };
        self.evaluate = function (array, pathToArray) {
            var inputArray, arrayArg = array, notifyAdded = false, notifyDeleted = false, notifyUnchanged = false, notifyChanged = false, addedItems = [], deletedItems = [];

            if (typeof array === "object" && pathToArray != undefined) {
                inputArray = parseDotnotation(array, pathToArray)
            } else {
                inputArray = arrayArg;
            };

             var newArray = CreateTrackingObject(inputArray, objKeys);


            for (var newHash in newArray) {
                if (newArray.hasOwnProperty(newHash)) {
                    if (orgArray.hasOwnProperty(newHash)  && orgArray[newHash].occurrences  === newArray[newHash].occurrences) {
                    } else {
                        if(!orgArray.hasOwnProperty(newHash)) { //new value
                            orgArray[newHash] = newArray[newHash];
                            addedItems.push(newArray[newHash].item);
                        } else { //has the value but must be more instances of them
                            var orgOccurrences = orgArray[newHash].occurrences;
                            for(var i =0;i< (newArray[newHash].occurrences - orgOccurrences);i++) {
                                orgArray[newHash].occurrences++;
                                addedItems.push(newArray[newHash].item);
                            }
                        }
                    }
                }

            };
            for (var oldHash in orgArray) {
                if (orgArray.hasOwnProperty(oldHash)) {
                    if (newArray.hasOwnProperty(oldHash) && orgArray[oldHash].occurrences  === newArray[oldHash].occurrences) {//item count same across arrays
                    } else {
                        if(!newArray.hasOwnProperty(oldHash)) {
                            deletedItems.push(orgArray[oldHash].item);
                            delete orgArray[oldHash];
                        } else {
                            var orgOccurrences = orgArray[oldHash].occurrences;
                            for(var i=0;i < (orgOccurrences - newArray[oldHash].occurrences);i++){
                                orgArray[oldHash].occurrences--;
                                deletedItems.push(orgArray[oldHash].item);
                            }
                        }

                    }
                }

            };

            if (addedItems.length > 0) {
                notifyAdded = true;
            }
            if (deletedItems.length > 0) {
                notifyDeleted = true;
            }
            if (notifyAdded || notifyDeleted) {
                notifyChanged = true;
            }
            if (!notifyAdded && !notifyDeleted) {
                notifyUnchanged = true;
            }
            function result() {
                var self = this;
                self.itemsDeleted = function (cb) {
                    if (notifyDeleted) {
                        cb(deletedItems);
                    }
                    return self;
                };
                self.itemsAdded = function (cb) {
                    if (notifyAdded) {
                        cb(addedItems);
                    }
                    return self;
                };
                self.changed = function (cb) {
                    if (notifyChanged) {
                        cb(arrayArg);
                    }
                    return self;
                };
                self.unchanged = function (cb) {
                    if (notifyUnchanged) {
                        cb(arrayArg);
                    }
                    return self;
                };

            }

            return new result();
        }

    };
    self.returnFunc = new tracker();
    return self.returnFunc;

};
exports = module.exports = ArrayTracker;
